#pragma once

// common data types
//
typedef char i8;
typedef unsigned char u8;
typedef short i16;
typedef unsigned short u16;
typedef long i32;
typedef unsigned long u32;
typedef int BOOL;
typedef unsigned char byte;
typedef unsigned short DACTYPE;         // type for DAC value

// variables types identifiers
//
typedef enum _Data_Types
{
    TYPE_BOOL   = 0,        // C bool value (int)
    TYPE_CHAR   = 1,        // signed char
    TYPE_UCHAR  = 2,        // unsgined char
    TYPE_BYTE   = 3,        // byte (unsigned char)
    TYPE_I16    = 4,        // signed short
    TYPE_U16    = 5,        // unsigned short
    TYPE_I32    = 6,        // int
    TYPE_U32    = 7,        // unsigned int
    TYPE_FLOAT  = 8,        // float
    TYPE_DOUBLE = 9,        // double
    TYPE_STRING = 10,       // zero terminated string
    TYPE_LAST   = 11,       // border
} Data_Types;


extern const u32 sizeofType[TYPE_LAST];
extern const char * const nameofType[];

// structure describing pixel configuration
//
typedef struct _PixelCfg
{
	byte lowTh: 3;          // low threshold (3 bits, low (0) is ACTIVE)
	byte highTh: 3;         // high threshold (3 bits, low (0) is ACTIVE)
	byte testBit: 1;        // test bit (1 bit, low (0) is ACTIVE)
	byte maskBit: 1;        // mask bit (1 bit, low (0) is ACTIVE)
} PixelCfg;

// structure describing one variable in HW
//
typedef struct _HwInfoItem
{
    Data_Types type;        // variable type
    u32 count;              // array size
    u32 flags;              // reserved e.g. interface specific config files
    const char *descr;      // item description
    void *data;             // pointer to variable of described type
} HwInfoItem;

#define MPX_HWINFO_CFGSAVE      0x0001      // value will be stored in config file
#define MPX_HWINFO_CANCHANGE    0x0002      // value can be changed

// macros for type conversions
//
#define LOWU16(l)           ((u16)((u32)(l) & 0xffff))
#define HIU16(l)            ((u16)((u32)(l) >> 16))
#define MAKEU32(a, b)       ((u32)(((u16)(a)) | (((u32)((u16)(b))) << 16)))

#define MPXCTRL_MAX_MSGLENGTH   4096        // maximum length of error message for some mpxCtrl functions
#define MPX_MAX_MSGLENGTH       4096        // maximum length of error message
#define MPX_MAX_PATH            512         // maximum length of path for file handling
#define MPX_MAX_CHBID           64          // max length of chipboard ID
#define MPX_MAX_IFACENAME       64          // max length of ifaceName name
#define MATRIX_SIZE             65536       // chip matrix size
#define FALSE   0                           // C bool values
#define TRUE    1

// stream buffer size (in i16) for n chips
#define I16STREAM_SIZE(n)       ((n * BYTES_PER_CHIP + 1) / 2)
#define I16STREAM_SIZE_MXR(n)   ((n * BYTES_PER_CHIP_MXR + 1) / 2)

#define DAC_COUNT               13
#define DAC_COUNT_MXR           15

// order of the Medipix DACs in array (u16[13]) for setMpxDACs
//
typedef enum _DACS_ORDER
{
    DELAYN = 0,
    DISC = 1,
    PREAMP = 2,
    SETDISC = 3,
    THS = 4,
    IKRUM = 5,
    ABUFFER = 6,
    VTHH = 7,
    VTHL = 8,
    VFBK = 9,
    VGND = 10,
    BIASLVDSTX = 11,
    REFLVDSTX = 12,
    IKRUMHALF = 13,
} DACS_ORDER;

typedef enum _DACS_CODES
{
	DELAYN_CODE = 1,
	DISC_CODE = 2,
	PREAMP_CODE = 3,
	SETDISC_CODE = 4,
	THS_CODE = 5,
	IKRUMHALF_CODE = 8,
	IKRUM_CODE = 7,
	ABUFFER_CODE = 14,
	VTHH_CODE = 12,
	VTHL_CODE = 11,
	VFBK_CODE = 10,
    VGND_CODE = 13,
	BIASLVDSTX_CODE = 6,
	REFLVDSTX_CODE = 9,
} DACS_CODES;


// order of the Medipix MXR DACs in array (u16[13]) for setMpxDACsMXR
// MXR has just 12 DACs, last item is not used (the size allows use same DAC functions for original chip as well as MXR)
//
typedef enum _DACS_ORDER_MXR
{
    MXR_IKRUM = 0,
    MXR_DISC = 1,
    MXR_PREAMP = 2,
    MXR_BUFFA = 3,
    MXR_BUFFB = 4,
    MXR_DELAYN = 5,
    MXR_THLFINE = 6,
    MXR_THLCOARSE = 7,
    MXR_THHFINE = 8,
    MXR_THHCOARSE = 9,   
    MXR_FBK = 10,
    MXR_GND = 11,
    MXR_THS = 12,
    MXR_BIASLVDS = 13,
    MXR_REFLVDS = 14,
} DACS_ORDER_MXR;

typedef enum _DACS_CODES_MXR
{
    MXR_THS_CODE = 1,
    MXR_BIASLVDS_CODE = 2,
    MXR_BUFFA_CODE = 3,
    MXR_BUFFB_CODE = 4,
    MXR_DELAYN_CODE = 9,
    MXR_THL_CODE = 6,
    MXR_PREAMP_CODE = 7,
    MXR_FBK_CODE = 10,
    MXR_DISC_CODE = 11,
    MXR_THH_CODE = 12,
    MXR_GND_CODE = 13,
    MXR_REFLVDS_CODE = 14,
    MXR_IKRUM_CODE = 15,
} DACS_CODES_MXR;